/*
 * FILE --- pid_ctrl.cc
 *
 * PURPOSE --- Source file for a class that implements a Proportional 
 * Integral Derivative (PID) controller. The derivative is obtained as
 * a simple difference between two succesive values of the input to the
 * controller and the integral is calculated using a 4'th order 
 * Runge-Kutta simulation routine. The user defines the PID gains and
 * the sampling time for the controller.
 *
 */

// Include Files
#include "pid_ctrl.hh"

// Define this macro if you want to use this utility without linking
// to the RCS Library
#ifdef NO_RCS
#include <stdio.h>            /* printf function */
#else
#include "rcs_prnt.hh"          /* rcs_print_ functions */
#endif


/*************************************************************/

PID_CTRL::PID_CTRL() : RK4_SIM(1, 1)
{
  // No error at start
  pid_ctrl_err = PID_CTRL_NO_ERROR;

  if (rk4_sim_err != RK4_SIM_NO_ERROR)
    {
      pid_ctrl_err = PID_CTRL_INIT_ERROR;
#ifdef NO_RCS
      printf("PID_CTRL: ERROR --- Not Enought Memory!!!\n");
      printf("The PID controller will not function correctly ...\n");
#else
      rcs_print("PID_CTRL: ERROR --- Not Enought Memory!!!\n");
      rcs_print("The PID controller will not function correctly ...\n");
#endif
    }
  
  // Set the sampling time to 0.01 and the initila time to 0.0
  init_pid(0.01);

  // Set all the gains to 0.0
  set_gains(0.0, 0.0, 0.0);
}

PID_CTRL::~PID_CTRL()
{
  
}

void PID_CTRL::init_pid(double _sample_time, double _initial_time = 0.0)
{
  sample_time = _sample_time;
  current_time = _initial_time;

  // Initialize the parameters of the integrator, RK4_SIM
  init_param(sample_time, current_time, 0);

  // There was no previous input
  old_input = 0.0;
  derivative = 0.0;
}

double PID_CTRL::control(const double input)
{
  // Find the integral of the input
  ctrl_input[0] = input;
  solve();
  
  // Find the derivative of the input
  derivative = (input - old_input)/sample_time;
  
  // Save the current input for the next time
  old_input = input;
  
  // Calculate and return the control output
  return( Kp*input + Ki*plant_state[0] + Kd*derivative );
}

void PID_CTRL::set_gains(double _Kp, double _Ki, double _Kd)
{
  Kp = _Kp;
  Ki = _Ki;
  Kd = _Kd;
}

void PID_CTRL::DERIV(const double t, const double *x, 
		     const double *u, double *xdot)
{
  
  // Implement inegrator G(s) = 1/s
  xdot[0] = u[0];    
  
}
